#: Directory of the item files within the container
PB_ITEM = '/item'
#: Directory of the task related JSON files within the container
PB_TASK = '/task'
#: Temporary (hard disk backed) temporary directory (auto cleaned upon exit)
PB_TMP = '/tmp'
#: Temporary fast (memory backed) temporary directory (auto cleaned upon exit)
#: Limited in size, currently 1.5G
PB_FAST = '/var/tmp/fast'

